#include "main.h"

Application::Application(void):
	mQuit(false)
{
}

Application::~Application(void)
{
	if (gMainMenu)
		delete gMainMenu;

	if (gGame)
		delete gGame;

	SDL_Quit();
}

bool Application::OnInit()
{
	if(SDL_Init(SDL_INIT_EVERYTHING) < 0)
		return false;

	gDisplay = SDL_SetVideoMode(600, 600, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	SDL_WM_SetCaption("Snake", NULL);

	if (!gDisplay)
		return false;

	gMainMenu = new MainMenu();
	if (!gMainMenu->OnInit())
		return false;

	mCurrentAppState = gMainMenu;

	return true;
}

void Application::OnUpdate()
{
	mCurrentAppState->OnUpdate();
}

void Application::OnEvent(SDL_Event Event)
{
	switch (Event.type)
	{
	case SDL_QUIT:
		mQuit = true;
		break;
	}

	mCurrentAppState->OnEvent(Event);
}

bool Application::Go()
{
	if (!OnInit())
		return false;

	SDL_Event Event;

    while(!mQuit)
	{
        while(SDL_PollEvent(&Event))
		{
			OnEvent(Event);
		}

		OnUpdate();
		OnDraw();
	}

	return true;
}

void Application::OnDraw()
{
	SDL_Rect rect;
	rect.x=0;
	rect.y=0;
	rect.w=600;
	rect.h=600;

	SDL_FillRect(gDisplay,&rect,0xffffffff);

	mCurrentAppState->OnDraw();

	SDL_Flip(gDisplay);
}

void Application::Quit()
{
	mQuit = true;
}

bool Application::NewGame()
{
	gGame = new Game();
	if (!gGame->OnInit(3,500))
		return false;

	mCurrentAppState = gGame;

	return true;
}

void Application::Loose()
{
	delete gGame;
	gGame = NULL;

	mCurrentAppState = gMainMenu;
}