#include "main.h"

Game::Game(void):
	mDirection(RIGHT), mNewDirection(RIGHT), mGrowing(false), mScore(0), mSnakeSkin(NULL), mMouseSkin(NULL)
{
}

Game::~Game(void)
{
	if (mSnakeSkin)
		SDL_FreeSurface(mSnakeSkin);

	if (mMouseSkin)
		SDL_FreeSurface(mMouseSkin);
}

bool Game::OnInit(unsigned int length, unsigned int interval)
{
	mSnakeSkin = SDL_LoadBMP("media/skin.bmp");

	if (!mSnakeSkin)
		return false;

	Node node;
	node.PosY=0;

	for (unsigned int i = 0; i<length; i++)
	{
		node.PosX=i;
		mSnake.push_back(node);
	}

	srand((unsigned)time(0));
	//mouse
	PlaceMouse(); //novietojam kaut kur peli
	mMouseSkin=SDL_LoadBMP("media/mouse.bmp");

	if (!mMouseSkin)
		return false;

	mInterval = interval;

	mLastTicks = SDL_GetTicks();

	return true;
}

void Game::OnUpdate()
{
	if (SDL_GetTicks() - mLastTicks > mInterval) //update time
	{
		mLastTicks = SDL_GetTicks();

		mDirection = mNewDirection;

		if (mGrowing)
			mGrowing=false;
		else
			mSnake.erase(mSnake.begin()); //idzeesham asti


		Node node;

		switch(mDirection)
		{
		case UP:
			node.PosX=mSnake.back().PosX;
			node.PosY=mSnake.back().PosY-1;
			break;
		case RIGHT:
			node.PosX=mSnake.back().PosX+1;
			node.PosY=mSnake.back().PosY;
			break;
		case DOWN:
			node.PosX=mSnake.back().PosX;
			node.PosY=mSnake.back().PosY+1;
			break;
		case LEFT:
			node.PosX=mSnake.back().PosX-1;
			node.PosY=mSnake.back().PosY;
			break;
		}

		mSnake.push_back(node);

		if (node.PosX<0 || node.PosX>=12 || //ja ieskrienam siena
			node.PosY<0 || node.PosY>=12)
		{
			gApp->Loose();
			return;
		}

		for(std::vector<Node>::iterator i = mSnake.begin(); i!=mSnake.end()-1; i++) //parbaudam, vai neieskrienam sevi
		{
			if (node.PosX==(*i).PosX && node.PosY==(*i).PosY)
			{
				gApp->Loose();
				return;
			}
		}

		if (node.PosX == mMouse.PosX && node.PosY == mMouse.PosY)
			EatMouse();
	}
}

void Game::OnEvent(SDL_Event Event)
{
	switch(Event.type)
	{
	case SDL_KEYDOWN:
		{
			switch(Event.key.keysym.sym)
			{
			case (SDLK_UP):
				if (mDirection != DOWN) mNewDirection = UP;
				break;
			case (SDLK_RIGHT):
				if (mDirection != LEFT) mNewDirection = RIGHT;
				break;
			case (SDLK_DOWN):
				if (mDirection != UP) mNewDirection = DOWN;
				break;
			case (SDLK_LEFT):
				if (mDirection != RIGHT) mNewDirection = LEFT;
				break;
			}
			break;
		}
	}
}

void Game::OnDraw()
{
	//zimejam peli
	SDL_Rect rect;
	rect.x=mMouse.PosX*50;
	rect.y=mMouse.PosY*50;
	rect.w=50;
	rect.h=50;

	SDL_BlitSurface(mMouseSkin, NULL, gDisplay, &rect);

	//zimejam cusku
	for(std::vector<Node>::iterator i = mSnake.begin(); i!=mSnake.end(); i++)
	{
		SDL_Rect rect;
		rect.x=(*i).PosX*50;
		rect.y=(*i).PosY*50;
		rect.w=50;
		rect.h=50;

		SDL_BlitSurface(mSnakeSkin, NULL, gDisplay, &rect);
	}
}

void Game::EatMouse()
{
	PlaceMouse();	

	mScore++;
	mInterval-=5;
	mGrowing = true;

	char caption[255];
	sprintf(caption, "Snake, score: %d", mScore);
	SDL_WM_SetCaption(caption, NULL);
}

void Game::PlaceMouse()
{
	bool placed=false;

	while(!placed) //cikls, lai pele neparaditos zem cuskas
	{
		mMouse.PosX=rand()%12;
		mMouse.PosY=rand()%12;

		placed=true;

		for(std::vector<Node>::iterator i = mSnake.begin(); i!=mSnake.end(); i++)
		{
			if (mMouse.PosX==(*i).PosX && mMouse.PosY==(*i).PosY) //vai pele atrdoas zem cuskas
			{
				placed=false;
				break;
			}
		}
	}
}